import click
import logging

@click.group()
@click.option('-v', '--verbose', count=True)
def main(verbose=0):
    if verbose < 1:
        level = logging.WARN
    elif verbose == 1:
        level = logging.INFO
    elif verbose >= 2:
        level = logging.DEBUG
    logging.basicConfig(level=level)

@click.command('s3:push')
@click.option('-b', '--bucket', required=True)
@click.option('-p', '--public', default=False, is_flag=True)
@click.option('--acl')
@click.option('--prefix')
@click.option('--expires', type=int, default=86400)
@click.argument('package')
def s3(package, bucket, acl, prefix, public, expires):
    from .s3 import push

    url = push(
        package_path=package,
        bucket=bucket,
        acl=acl,
        prefix=prefix,
        public=public,
        expires=expires,
    )
    click.echo(url)

main.add_command(s3)



from contextlib import contextmanager
from pyaxmlparser import APK
import zipfile

@contextmanager
def open_apk(path):
    apk = APK(path)
    yield apk

def is_apk(path):
    try:
        with open(path, 'rb') as fp:
            with zipfile.ZipFile(fp) as zip:
                return 'AndroidManifest.xml' in zip.namelist()
    except Exception:
        return False



import attr
import boto3
import importlib.resources
import mimetypes
import os.path
import typing

from .apk import is_apk, open_apk
from .ipa import is_ipa, open_ipa
from .templates import render_template

def push(
    package_path,
    bucket,
    *,
    acl=None,
    public=False,
    expires=86400,
    prefix=None,
):
    if not acl:
        acl = 'public-read' if public else 'private'

    if not prefix:
        prefix = os.path.splitext(os.path.basename(package_path))[0]

    s3 = S3(
        client=boto3.client('s3'),
        bucket=bucket,
        acl=acl,
        public=public,
        expires=expires,
    )

    if is_ipa(package_path):
        with open_ipa(package_path) as ipa:
            ctx = push_ipa(ipa, s3, prefix)

    elif is_apk(package_path):
        with open_apk(package_path) as apk:
            ctx = push_apk(apk, s3, prefix)

    else:
        raise ValueError('unrecognized package')

    html_tmpl = importlib.resources.read_text(__package__, 'html.jinja2')
    html = render_template(html_tmpl, context=ctx)
    url = s3.put_object(
        f'{prefix}.html',
        html.encode('utf8'),
        'text/html',
    )
    return url

@attr.s(auto_attribs=True)
class S3:
    client: typing.Any
    bucket: str
    acl: str
    public: bool
    expires: int

    def put_object(self, path, data, content_type):
        self.client.put_object(
            Bucket=self.bucket,
            Key=path,
            Body=data,
            ACL=self.acl,
            ContentType=content_type,
        )

        url = self.client.generate_presigned_url(
            'get_object',
            Params=dict(
                Bucket=self.bucket,
                Key=path,
            ),
            ExpiresIn=self.expires,
        )
        if self.public:
            i = url.index('?')
            url = url[:i]
        return url

def push_ipa(ipa, s3, prefix):
    image_url = None
    icon = ipa.find_best_icon(512)
    if icon is not None:
        with ipa.open_asset(icon.name) as fp:
            image_url = s3.put_object(f'{prefix}.png', fp, 'image/png')

    with open(ipa.filename, 'rb') as fp:
        ipa_url = s3.put_object(f'{prefix}.ipa', fp, 'application/zip')

    manifest_tmpl = importlib.resources.read_text(
        __package__, 'ipa_manifest.jinja2')
    manifest = render_template(manifest_tmpl, context=dict(
        ipa=ipa,
        ipa_url=ipa_url,
        image_url=image_url,
    ))
    manifest_url = s3.put_object(
        f'{prefix}.plist',
        manifest.encode('utf8'),
        'text/xml',
    )

    return dict(
        package_type='ipa',
        app_id=ipa.id,
        display_name=ipa.display_name,
        version_name=ipa.short_version,
        version_code=ipa.version,
        minimum_os_version=ipa.minimum_os_version,
        image_url=image_url,
        ipa_url=ipa_url,
        manifest_url=manifest_url,
    )

def push_apk(apk, s3, prefix):
    image_url = None
    icon = apk.get_app_icon(512)
    if icon and not icon.endswith('.xml'):
        image_data = apk.get_file(icon)
        image_ext = os.path.splitext(icon)[1]
        image_type, _ = mimetypes.guess_type(icon, strict=False)
        image_url = s3.put_object(f'{prefix}{image_ext}', image_data, image_type)

    with open(apk.filename, 'rb') as fp:
        apk_url = s3.put_object(
            f'{prefix}.apk', fp, 'application/vnd.android.package-archive')

    return dict(
        package_type='apk',
        app_id=apk.package,
        display_name=apk.application,
        version_name=apk.version_name,
        version_code=apk.version_code,
        image_url=image_url,
        apk_url=apk_url,
    )



import attr
from contextlib import contextmanager
import plistlib
import posixpath
import re
import typing
import zipfile

@attr.s(auto_attribs=True)
class Icon:
    size: int
    name: str

@attr.s(auto_attribs=True)
class IPA:
    filename: str
    zip: zipfile.ZipFile
    app_path: str

    id: str
    display_name: str
    short_version: str
    version: str
    minimum_os_version: str
    icons: typing.Dict[int, Icon]
    plist: typing.Dict[str, typing.Any]

    @contextmanager
    def open_asset(self, name):
        with self.zip.open(f'{self.app_path}/{name}') as fp:
            yield fp

    def find_best_icon(self, target_size=1024):
        best_icon = best_dist = None
        for size, icon in self.icons.items():
            dist = abs(target_size - size)
            if best_dist is None or dist < best_dist:
                best_icon = icon
                best_dist = dist
        return best_icon

def is_ipa(path):
    try:
        with open(path, 'rb') as fp:
            with zipfile.ZipFile(fp) as zip:
                get_app_path(zip)
                return True
    except Exception:
        return False

@contextmanager
def open_ipa(path):
    with open(path, 'rb') as fp:
        with zipfile.ZipFile(fp) as zip:
            app_path = get_app_path(zip)
            with zip.open(f'{app_path}/Info.plist') as plist_fp:
                plist = plistlib.load(plist_fp)
            meta = parse_metadata(plist, zip, app_path)
            ipa = IPA(
                filename=path,
                zip=zip,
                app_path=app_path,
                plist=plist,
                **meta,
            )
            yield ipa

def get_app_path(zip):
    plist_re = re.compile(r'Payload/[^/]+\.app/Info\.plist')
    for path in zip.namelist():
        if plist_re.match(path):
            return posixpath.dirname(path)
    raise ValueError('unable to find Info.plist')

def parse_metadata(plist, zip, app_path):
    icons = {}
    icons.update(parse_icon_metadata(plist, zip, app_path, 'CFBundleIcons'))
    icons.update(parse_icon_metadata(plist, zip, app_path, 'CFBundleIcons~ipad'))

    return dict(
        id=plist['CFBundleIdentifier'],
        display_name=plist['CFBundleDisplayName'],
        short_version=plist['CFBundleShortVersionString'],
        version=plist['CFBundleVersion'],
        minimum_os_version=plist.get('MinimumOSVersion'),
        icons=icons,
    )

icon_re = re.compile(
    r'''
    (?P<w>\d+\.?\d*)
    x
    (?P<h>\d+\.?\d*)
    (?:
        @
        (?P<r>\d+)
    )?
    (?:~ipad)?
    \.png
    ''',
    re.VERBOSE,
)

def parse_icon_metadata(plist, zip, app_path, key):
    icons = {}
    files = (
        plist
        .get(key, {})
        .get('CFBundlePrimaryIcon', {})
        .get('CFBundleIconFiles', [])
    )
    for icon_prefix in files:
        for path in zip.namelist():
            if not path.startswith(f'{app_path}/{icon_prefix}'):
                continue
            name = posixpath.basename(path)
            m = icon_re.search(name)
            if not m:
                continue
            r = int(m.group('r') or 1)
            icon = Icon(
                size=int(float(m['w']) * r),
                name=name,
            )
            icons[icon.size] = icon
    return icons



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['airsupply']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=19.3.0,<20.0.0',
 'boto3>=1.11.8,<2.0.0',
 'click>=7.0,<8.0',
 'jinja2>=2.10.3,<3.0.0',
 'pyaxmlparser>=0.3.24,<0.4.0']

entry_points = \
{'console_scripts': ['airsupply = airsupply.cli:main']}

setup_kwargs = {
    'name': 'airsupply',
    'version': '0.2.0',
    'description': 'Manage OTA distribution for IPA and APK files.',
    'long_description': None,
    'author': 'Michael Merickel',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
